/**
 * Copyright (C) 2011 Agrosense <info@cloudfarming.nl>
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.stock.project.visual;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.util.List;
import nl.cloudfarming.client.crop.model.BeanFactory;
import nl.cloudfarming.client.crop.model.CropBean;

/**
 * A controller which displays Beans.
 * It allows to add and remove Beans
 * 
 * Will fire PropertyChange events
 * If a row is added a PropertyChange with property name PROP_ADD_ROW will be fired
 * It contains null as the old value and the newly added T extends Bean as newValue
 * If the selection changes between rows a PropertyChange with the property name PROP_SELECTION will be thrown
 * It contains the previous selected row as the oldValue and the newly selected row as newValue 
 * @author Merijn Zengers
 */
public class ListController<T extends CropBean> implements ActionListener, PropertyChangeListener {

    private String header;
    private String label;
    private ListPanel<T> rowPanel;
    private PropertyChangeSupport propertyChangeSupport;
    private BeanFactory beanFactory;
    
    public static final String PROP_SELECTION = "selection";
    public static final String PROP_ADD_ROW = "add";
    public static final String PROP_REMOVE_ROW = "remove";

    /**
     * 
     * @param beanFactory Factory to retrieve new bean from when the add button is pushed
     * @param beans Existing Beans to initialize the list with
     * @param header Header of this element
     * @param label ColumnHeader of the listPanel listing all beans
     */
    public ListController(BeanFactory beanFactory, List<T> beans, String header, String label) {
        this.header = header;
        this.label = label;
        this.beanFactory = beanFactory;
        init(beans);
    }

    private void init(List<T> beans) {
        rowPanel = new ListPanel<>(beans, header, label);
        rowPanel.addPropertyChangeListener(this);
        rowPanel.getAddButton().addActionListener(this);
    }

    /**
     * Sets the beans displayed in the ListPanel
     * For each bean a row will be added to the ListPanel
     * @param beans the beans to display as rows
     */
    public void setBeans(List<T> beans) {
        rowPanel.setBeans(beans);
    }
    
    public void addBean(T bean){
        rowPanel.addRow(bean);
    }
    
    public void setEnabled(boolean enabled){
        rowPanel.getAddButton().setEnabled(enabled);
    }
    
    /**
     * Gets the selected Bean from the rowPanel
     * @return The selected Bean or null if none is selected
     */
    public T getSelectedBean(){
        return (T) (getComponent().getSelectedRow() != null ? getComponent().getSelectedRow().getBean() : null);
    }
    
    /**
     * Sets the selected row based on the bean the row contains
     * If the bean is not contained in the ListPanel nothing will be set
     * @param bean the bean to select
     */
    public void setSelectedBean(T bean){
        Row selected = getComponent().getRow(bean);
        if(selected != null){
            getComponent().setSelectedRow(selected);
        }
    }

    /**
     * Gets the ListPanel this controller is controlling
     * @return the ListPanel
     */
    public ListPanel getComponent() {
        return rowPanel;
    }

    @Override
    public void actionPerformed(ActionEvent ae) {
        if (ae.getSource().equals(rowPanel.getAddButton())) {
            T newRow = (T) beanFactory.create();

            rowPanel.addRow(newRow);
            getPropertyChangeSupport().firePropertyChange(PROP_ADD_ROW, null, newRow);
        }
    }

    @Override
    public void propertyChange(PropertyChangeEvent pce) {
        switch (pce.getPropertyName()) {
            case ListPanel.PROP_SELECTION:
                CropBean old = pce.getOldValue() == null ? null : ((Row) pce.getOldValue()).getBean();
                CropBean newBean = pce.getNewValue() == null ? null : ((Row) pce.getNewValue()).getBean();
                getPropertyChangeSupport().firePropertyChange(PROP_SELECTION, old, newBean);
                break;
            case ListPanel.PROP_ROW:
                getPropertyChangeSupport().firePropertyChange(PROP_REMOVE_ROW, pce.getOldValue(), pce.getNewValue());
                getPropertyChangeSupport().firePropertyChange(PROP_SELECTION, pce.getOldValue(), null);
                break;
        }
    }

    public void addPropertyChangeListener(PropertyChangeListener pcl) {
        getPropertyChangeSupport().addPropertyChangeListener(pcl);
    }

    public void addPropertyChangeListener(String propertyName, PropertyChangeListener pcl) {
        getPropertyChangeSupport().addPropertyChangeListener(propertyName, pcl);
    }

    public void removePropertyChangeListener(PropertyChangeListener pcl) {
        getPropertyChangeSupport().removePropertyChangeListener(pcl);
    }

    public void removePropertyChangeListener(String propertyName, PropertyChangeListener pcl) {
        getPropertyChangeSupport().removePropertyChangeListener(propertyName, pcl);
    }

    PropertyChangeSupport getPropertyChangeSupport() {
        if (propertyChangeSupport == null) {
            propertyChangeSupport = new PropertyChangeSupport(this);
        }
        return propertyChangeSupport;
    }
}
